/*
 * Copyright (C) 2008 Luke Kenneth Casson Leighton <lkcl@lkcl.net>
 * Copyright (C) 2008 Alp Toker <alp@atoker.com>
 * Copyright (C) 2008 Apple Inc.
 */

#include "config.h"

#include <wtf/GetPtr.h>
#include "CSSRule.h"
#include "CSSValue.h"
#include "CanvasPixelArray.h"
#include "DOMWindow.h"
#include "DocumentFragment.h"
#include "Element.h"
#include "Event.h"
#include "GDOMBinding.h"
#include "GDOMObject.h"
#include "HTMLCollection.h"
#include "Node.h"
#include "Range.h"
#include "StyleSheet.h"
#include "Text.h"
#include "gdom/GStringConvert.h"
#include "gdom/GdomDOMObject.h"
#include "gdom/GdomDocumentFragment.h"
#include "gdom/GdomNode.h"
#include "gdom/GdomRange.h"
#include "gdom/PrivateGdomDOMObject.h"
#include "gdom/PrivateGdomDocumentFragment.h"
#include "gdom/PrivateGdomNode.h"
#include "gdom/PrivateGdomRange.h"
#include "gdom/PrivateIntGdomDocumentFragment.h"
#include "gdom/PrivateIntGdomNode.h"
#include "gdom/PrivateIntGdomRange.h"
#include "glib-object.h"

namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore

namespace WebCore {
gpointer toGDOM(WebCore::Range* obj)
{
    return WebCore::cacheDOMObject<WebCore::Range, GDOMRange>(obj);
}
} /* namespace WebCore */
extern "C" {

GDOMRange::GDOMRange(PassRefPtr<WebCore::Range> impl)
    : GDOMObject()
    , m_impl(impl)
{
}

GDOMRange::~GDOMRange()
{
    m_impl.get()->deref();
    WebCore::GDOMScriptInterpreter::forgetDOMObject(m_impl.get());

}

void
gdom_range_set_start (GdomRange *thiz, GdomNode * ref_node, glong offset)
{
    g_return_if_fail (thiz);
    WebCore::Range * item = (gdom_range_get_private(thiz));
    g_return_if_fail (item);
    g_return_if_fail (ref_node);
    WebCore::Node * _g_ref_node = (gdom_node_get_private(ref_node));
    g_return_if_fail (_g_ref_node);
    WebCore::ExceptionCode ec = 0;
    item->setStart(_g_ref_node, offset, ec);

}

void
gdom_range_set_end (GdomRange *thiz, GdomNode * ref_node, glong offset)
{
    g_return_if_fail (thiz);
    WebCore::Range * item = (gdom_range_get_private(thiz));
    g_return_if_fail (item);
    g_return_if_fail (ref_node);
    WebCore::Node * _g_ref_node = (gdom_node_get_private(ref_node));
    g_return_if_fail (_g_ref_node);
    WebCore::ExceptionCode ec = 0;
    item->setEnd(_g_ref_node, offset, ec);

}

void
gdom_range_set_start_before (GdomRange *thiz, GdomNode * ref_node)
{
    g_return_if_fail (thiz);
    WebCore::Range * item = (gdom_range_get_private(thiz));
    g_return_if_fail (item);
    g_return_if_fail (ref_node);
    WebCore::Node * _g_ref_node = (gdom_node_get_private(ref_node));
    g_return_if_fail (_g_ref_node);
    WebCore::ExceptionCode ec = 0;
    item->setStartBefore(_g_ref_node, ec);

}

void
gdom_range_set_start_after (GdomRange *thiz, GdomNode * ref_node)
{
    g_return_if_fail (thiz);
    WebCore::Range * item = (gdom_range_get_private(thiz));
    g_return_if_fail (item);
    g_return_if_fail (ref_node);
    WebCore::Node * _g_ref_node = (gdom_node_get_private(ref_node));
    g_return_if_fail (_g_ref_node);
    WebCore::ExceptionCode ec = 0;
    item->setStartAfter(_g_ref_node, ec);

}

void
gdom_range_set_end_before (GdomRange *thiz, GdomNode * ref_node)
{
    g_return_if_fail (thiz);
    WebCore::Range * item = (gdom_range_get_private(thiz));
    g_return_if_fail (item);
    g_return_if_fail (ref_node);
    WebCore::Node * _g_ref_node = (gdom_node_get_private(ref_node));
    g_return_if_fail (_g_ref_node);
    WebCore::ExceptionCode ec = 0;
    item->setEndBefore(_g_ref_node, ec);

}

void
gdom_range_set_end_after (GdomRange *thiz, GdomNode * ref_node)
{
    g_return_if_fail (thiz);
    WebCore::Range * item = (gdom_range_get_private(thiz));
    g_return_if_fail (item);
    g_return_if_fail (ref_node);
    WebCore::Node * _g_ref_node = (gdom_node_get_private(ref_node));
    g_return_if_fail (_g_ref_node);
    WebCore::ExceptionCode ec = 0;
    item->setEndAfter(_g_ref_node, ec);

}

void
gdom_range_collapse (GdomRange *thiz, gboolean to_start)
{
    g_return_if_fail (thiz);
    WebCore::Range * item = (gdom_range_get_private(thiz));
    g_return_if_fail (item);
    WebCore::ExceptionCode ec = 0;
    item->collapse(to_start, ec);

}

void
gdom_range_select_node (GdomRange *thiz, GdomNode * ref_node)
{
    g_return_if_fail (thiz);
    WebCore::Range * item = (gdom_range_get_private(thiz));
    g_return_if_fail (item);
    g_return_if_fail (ref_node);
    WebCore::Node * _g_ref_node = (gdom_node_get_private(ref_node));
    g_return_if_fail (_g_ref_node);
    WebCore::ExceptionCode ec = 0;
    item->selectNode(_g_ref_node, ec);

}

void
gdom_range_select_node_contents (GdomRange *thiz, GdomNode * ref_node)
{
    g_return_if_fail (thiz);
    WebCore::Range * item = (gdom_range_get_private(thiz));
    g_return_if_fail (item);
    g_return_if_fail (ref_node);
    WebCore::Node * _g_ref_node = (gdom_node_get_private(ref_node));
    g_return_if_fail (_g_ref_node);
    WebCore::ExceptionCode ec = 0;
    item->selectNodeContents(_g_ref_node, ec);

}


/* XXX TODO: skipping CompareHow function gdom_range_compare_boundary_points */

void
gdom_range_delete_contents (GdomRange *thiz)
{
    g_return_if_fail (thiz);
    WebCore::Range * item = (gdom_range_get_private(thiz));
    g_return_if_fail (item);
    WebCore::ExceptionCode ec = 0;
    item->deleteContents(ec);

}

GdomDocumentFragment *
gdom_range_extract_contents (GdomRange *thiz)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::Range * item = (gdom_range_get_private(thiz));
    g_return_val_if_fail (item, 0);
    WebCore::ExceptionCode ec = 0;
    PassRefPtr<WebCore::DocumentFragment> g_res = WTF::getPtr(item->extractContents(ec));
    GdomDocumentFragment * res = static_cast<GdomDocumentFragment *>(WebCore::toGDOM(g_res.get()));
    return res;

}

GdomDocumentFragment *
gdom_range_clone_contents (GdomRange *thiz)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::Range * item = (gdom_range_get_private(thiz));
    g_return_val_if_fail (item, 0);
    WebCore::ExceptionCode ec = 0;
    PassRefPtr<WebCore::DocumentFragment> g_res = WTF::getPtr(item->cloneContents(ec));
    GdomDocumentFragment * res = static_cast<GdomDocumentFragment *>(WebCore::toGDOM(g_res.get()));
    return res;

}

void
gdom_range_insert_node (GdomRange *thiz, GdomNode * new_node)
{
    g_return_if_fail (thiz);
    WebCore::Range * item = (gdom_range_get_private(thiz));
    g_return_if_fail (item);
    g_return_if_fail (new_node);
    WebCore::Node * _g_new_node = (gdom_node_get_private(new_node));
    g_return_if_fail (_g_new_node);
    WebCore::ExceptionCode ec = 0;
    item->insertNode(_g_new_node, ec);

}

void
gdom_range_surround_contents (GdomRange *thiz, GdomNode * new_parent)
{
    g_return_if_fail (thiz);
    WebCore::Range * item = (gdom_range_get_private(thiz));
    g_return_if_fail (item);
    g_return_if_fail (new_parent);
    WebCore::Node * _g_new_parent = (gdom_node_get_private(new_parent));
    g_return_if_fail (_g_new_parent);
    WebCore::ExceptionCode ec = 0;
    item->surroundContents(_g_new_parent, ec);

}

GdomRange *
gdom_range_clone_range (GdomRange *thiz)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::Range * item = (gdom_range_get_private(thiz));
    g_return_val_if_fail (item, 0);
    WebCore::ExceptionCode ec = 0;
    PassRefPtr<WebCore::Range> g_res = WTF::getPtr(item->cloneRange(ec));
    GdomRange * res = static_cast<GdomRange *>(WebCore::toGDOM(g_res.get()));
    return res;

}

gchar *
gdom_range_to_string (GdomRange *thiz)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::Range * item = (gdom_range_get_private(thiz));
    g_return_val_if_fail (item, 0);
    WebCore::ExceptionCode ec = 0;
    gchar * res = _G_GET_UTF8(item->toString(ec));
    return res;

}

void
gdom_range_detach (GdomRange *thiz)
{
    g_return_if_fail (thiz);
    WebCore::Range * item = (gdom_range_get_private(thiz));
    g_return_if_fail (item);
    WebCore::ExceptionCode ec = 0;
    item->detach(ec);

}

GdomDocumentFragment *
gdom_range_create_contextual_fragment (GdomRange *thiz, gchar * html)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::Range * item = (gdom_range_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (html, 0);
    WebCore::String _g_html = WebCore::String::fromUTF8(html);
    WebCore::ExceptionCode ec = 0;
    PassRefPtr<WebCore::DocumentFragment> g_res = WTF::getPtr(item->createContextualFragment(_g_html, ec));
    GdomDocumentFragment * res = static_cast<GdomDocumentFragment *>(WebCore::toGDOM(g_res.get()));
    return res;

}

gboolean
gdom_range_intersects_node (GdomRange *thiz, GdomNode * ref_node)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::Range * item = (gdom_range_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (ref_node, 0);
    WebCore::Node * _g_ref_node = (gdom_node_get_private(ref_node));
    g_return_val_if_fail (_g_ref_node, 0);
    WebCore::ExceptionCode ec = 0;
    gboolean res = item->intersectsNode(_g_ref_node, ec);
    return res;

}

gshort
gdom_range_compare_node (GdomRange *thiz, GdomNode * ref_node)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::Range * item = (gdom_range_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (ref_node, 0);
    WebCore::Node * _g_ref_node = (gdom_node_get_private(ref_node));
    g_return_val_if_fail (_g_ref_node, 0);
    WebCore::ExceptionCode ec = 0;
    gshort res = item->compareNode(_g_ref_node, ec);
    return res;

}

gshort
gdom_range_compare_point (GdomRange *thiz, GdomNode * ref_node, glong offset)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::Range * item = (gdom_range_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (ref_node, 0);
    WebCore::Node * _g_ref_node = (gdom_node_get_private(ref_node));
    g_return_val_if_fail (_g_ref_node, 0);
    WebCore::ExceptionCode ec = 0;
    gshort res = item->comparePoint(_g_ref_node, offset, ec);
    return res;

}

gboolean
gdom_range_is_point_in_range (GdomRange *thiz, GdomNode * ref_node, glong offset)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::Range * item = (gdom_range_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (ref_node, 0);
    WebCore::Node * _g_ref_node = (gdom_node_get_private(ref_node));
    g_return_val_if_fail (_g_ref_node, 0);
    WebCore::ExceptionCode ec = 0;
    gboolean res = item->isPointInRange(_g_ref_node, offset, ec);
    return res;

}

} /* extern "C" */


#include "config.h"
#include "glib-object.h"
#include <webkit/webkitdefines.h>
#include <webkit/webkitprivate.h> 
#include "DOMWindow.h" /* yukk: TODO - make this optional */
#include "Node.h" /* yukk: TODO - make this optional */
#include "NamedNodeMap.h" /* yukk: TODO - make this optional */
#include "CanvasPattern.h" /* yukk: TODO - make this optional */
#include "CanvasGradient.h" /* yukk: TODO - make this optional */
#include "CanvasStyle.h" /* yukk: TODO - make this optional */
#include "CanvasRenderingContext2D.h" /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomDOMObject.h> /* yukk: TODO - make this optional */
#include <gdom/GdomEventTargetNode.h> /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomEventTargetNode.h> /* yukk: TODO - make this optional on EventTarget*/
#include "Range.h"
#include <gdom/GdomRange.h>
#include <gdom/PrivateGdomRange.h>
#include <gdom/PrivateIntGdomRange.h>
#include <gdom/GdomRangeprops.h>

#include "gdom/GStringConvert.h"

#include "CanvasPixelArray.h"
#include "Event.h"
#include "Text.h"
#include "CSSValue.h"
#include "CSSRule.h"
#include "HTMLCollection.h"
#include "StyleSheet.h"
#include "Document.h"
#include "webkitmarshal.h"
#include <gdom/GdomEvent.h>



namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore



G_BEGIN_DECLS

G_DEFINE_TYPE(GdomRange, gdom_range, GDOM_TYPE_DOM_OBJECT);

/*
struct _GdomRangePrivate {
    class WebCore::Range *item;
};

#define GDOM_RANGE_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), GDOM_TYPE_RANGE, GdomRangePrivate))
*/
G_END_DECLS
WebCore::Range *
gdom_range_get_private(GdomRange *request)
{
    GDOMRange *res;
    res = static_cast<GDOMRange*>(gdom_dom_object_get_private(GDOM_DOM_OBJECT(request)));
    return static_cast<WebCore::Range*>(res->impl());
}

G_BEGIN_DECLS

enum {
    BROWSER_EVENT,
    LAST_SIGNAL
};

static guint event_signals[LAST_SIGNAL] = { 0, };

enum {
    PROP_0,
    PROP_START_CONTAINER,
    PROP_START_OFFSET,
    PROP_END_CONTAINER,
    PROP_END_OFFSET,
    PROP_COLLAPSED,
    PROP_COMMON_ANCESTOR_CONTAINER,
    PROP_TEXT,
};


static void gdom_range_finalize(GObject* object)
{
    /*GdomRange* request = GDOM_RANGE(object);*/
    /*GdomRangePrivate* priv = request->priv;*/

    /*GObjectClass *parent_class = g_type_class_peek (g_type_peek_parent (GDOM_TYPE_DOM_OBJECT));*/

    /* Free any memory or close any files */

    /*parent_class-> finalize (object);*/

    G_OBJECT_CLASS(gdom_range_parent_class)->finalize(object);
}

static void gdom_range__set_property_fn(GObject * object, guint prop_id,
                     const GValue * value,
                     GParamSpec * pspec)
{
    GdomRange *self = GDOM_RANGE(object);
    WebCore::Range * gself = (gdom_range_get_private(self));
    switch (prop_id) {
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_range__get_property_fn(GObject * object, guint prop_id,
                     GValue * value,
                     GParamSpec * pspec)
{
    GdomRange *self = GDOM_RANGE(object);
    WebCore::Range * gself = (gdom_range_get_private(self));
    switch (prop_id) {
    case PROP_START_CONTAINER:
    {
         WebCore::ExceptionCode ec = 0;
         PassRefPtr<WebCore::Node> ptr = gself->startContainer(ec);
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_START_OFFSET:
    {
         WebCore::ExceptionCode ec = 0;
         g_value_set_long (value, gself->startOffset(ec));
         break;
    }
    case PROP_END_CONTAINER:
    {
         WebCore::ExceptionCode ec = 0;
         PassRefPtr<WebCore::Node> ptr = gself->endContainer(ec);
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_END_OFFSET:
    {
         WebCore::ExceptionCode ec = 0;
         g_value_set_long (value, gself->endOffset(ec));
         break;
    }
    case PROP_COLLAPSED:
    {
         WebCore::ExceptionCode ec = 0;
         g_value_set_boolean (value, gself->collapsed(ec));
         break;
    }
    case PROP_COMMON_ANCESTOR_CONTAINER:
    {
         WebCore::ExceptionCode ec = 0;
         PassRefPtr<WebCore::Node> ptr = gself->commonAncestorContainer(ec);
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_TEXT:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->text()));
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_range_class_init(GdomRangeClass* requestClass)
{
    GObjectClass *g_klass = G_OBJECT_CLASS (requestClass);
    g_klass->finalize = gdom_range_finalize;
    g_klass->set_property = gdom_range__set_property_fn;
    g_klass->get_property = gdom_range__get_property_fn;

    g_object_class_install_property (g_klass, PROP_START_CONTAINER,
            g_param_spec_object ("start_container", /* name */
                "range_start_container", /* short description */
                "read-only  GdomNode * Range.start_container", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_NODE, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_START_OFFSET,
            g_param_spec_long ("start_offset", /* name */
                "range_start_offset", /* short description */
                "read-only  glong Range.start_offset", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_END_CONTAINER,
            g_param_spec_object ("end_container", /* name */
                "range_end_container", /* short description */
                "read-only  GdomNode * Range.end_container", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_NODE, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_END_OFFSET,
            g_param_spec_long ("end_offset", /* name */
                "range_end_offset", /* short description */
                "read-only  glong Range.end_offset", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_COLLAPSED,
            g_param_spec_boolean ("collapsed", /* name */
                "range_collapsed", /* short description */
                "read-only  gboolean Range.collapsed", /* longer - could do with some extra doc stuff here */
                 FALSE, /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_COMMON_ANCESTOR_CONTAINER,
            g_param_spec_object ("common_ancestor_container", /* name */
                "range_common_ancestor_container", /* short description */
                "read-only  GdomNode * Range.common_ancestor_container", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_NODE, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_TEXT,
            g_param_spec_string ("text", /* name */
                "range_text", /* short description */
                "read-only  gchar * Range.text", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READABLE));





    /*g_type_class_remove_private(requestClass, sizeof(GdomRangePrivate));*/
}

static void gdom_range_init(GdomRange* request)
{
    /*GdomRangePrivate* priv = GDOM_RANGE_GET_PRIVATE(request);
    request->priv = priv;*/
}

} /* extern "C" */

